module INSTRUCTION_REGISTER #(
  parameter BITS = 8
)(
  input                  i_clk,
  input                  i_rst,
  input                  i_lst,
  input                  i_prg,
  input      [2:0]       i_flg,
  input      [BITS-1:0]  i_bus,
  input                  i_bus_active,
  output                 o_hlt,
  output                 o_ce,
  output                 o_co,
  output                 o_j,
  output                 o_mi,
  output                 o_ii,
  output                 o_ro,
  output                 o_ri,
  output                 o_ai,
  output                 o_ao,
  output                 o_bi,
  output                 o_bo,
  output                 o_di,
  output                 o_do,
  output                 o_oi,
  output                 o_oo,
  output                 o_eo,
  output                 o_e0,
  output                 o_e1,
  output                 o_e2,
  output                 o_ec,
  output                 o_po,
  output                 o_x1,
  output                 o_x2,
  output reg [7:0]       o_ir
);
reg          [7:0]       ir;
reg          [14:0]      addr;
reg          [3:0]       step;

always @(posedge i_clk || i_rst) begin
  if (i_rst) begin
    ir = 8'd0;
    step = 4'd0;
  end else begin
    step = (step < 4'd5) ? step + 4'd1 : 4'd0;
    if (o_ii)
      ir = (i_bus_active) ? i_bus[7:0] : 8'b0;
  end
end

always @(ir || i_lst || i_prg) begin
  if (i_lst) begin
    if (i_prg)
      o_ir = 8'hFE;
    else
      o_ir = 8'hFC;
  end else
    o_ir = ir;
end

CONTROL ctl_rom (
  .i_clk  (i_clk),
  .i_addr (addr),
  .o_hlt  (o_hlt),
  .o_ce   (o_ce),
  .o_co   (o_co),
  .o_j    (o_j),
  .o_mi   (o_mi),
  .o_ii   (o_ii),
  .o_ro   (o_ro),
  .o_ri   (o_ri),
  .o_ai   (o_ai),
  .o_ao   (o_ao),
  .o_bi   (o_bi),
  .o_bo   (o_bo),
  .o_di   (o_di),
  .o_do   (o_do),
  .o_oi   (o_oi),
  .o_oo   (o_oo),
  .o_eo   (o_eo),
  .o_e0   (o_e0),
  .o_e1   (o_e1),
  .o_e2   (o_e2),
  .o_ec   (o_ec),
  .o_po   (o_po),
  .o_x1   (o_x1),
  .o_x2   (o_x2)
);

always @(negedge i_clk) begin //o_ir || step || i_flg) begin
  addr = {o_ir,step,i_flg};
end

endmodule
